/* SmoothWall install program.
 *
 * By The SmoothWall Team.
 * 
 * This program is distributed under the terms of the GNU General Public
 * Licence.  See the file COPYING for details.
 *
 * filename: config.c
 * Write the config and get password stuff. */


#include "install.h"

extern FILE *flog;
extern char *log;

int writeconfigs(struct devparams *dp, struct networkparams *np)
{
	FILE *f;
	char commandstring[STRING_SIZE];
	char password[STRING_SIZE];
	char devnode[STRING_SIZE];

	/* main contains just the hostname. */		
	if (!(f = fopen("/harddisk/etc/sysconfig/main", "w")))
	{
		errorbox("Unable to write /etc/sysconfig/main.");
		return 0;
	}
	fprintf(f, "HOSTNAME=smoothwall\n");
	fclose(f);
	
	/* network settings. */
	if (!(f = fopen("/harddisk/etc/sysconfig/network", "w")))
	{
		errorbox("Unable to write /etc/sysconfig/network.");
		return 0;
	}
	fprintf(f, "DRIVER='%s'\n", np->modulename);
	fprintf(f, "IPADDR=%s\n", np->address);
	fprintf(f, "NETADDR=%s\n", np->netaddress);
	fprintf(f, "NETMASK=%s\n", np->netmask);
	fclose(f);
	
	/* hosts. */
	if (!(f = fopen("/harddisk/etc/hosts", "w")))
	{
		errorbox("Unable to write /etc/hosts.");
		return 0;
	}
	fprintf(f, "127.0.0.1\tlocalhost\n");
	fprintf(f, "%s\tsmoothwall\n", np->address);
	fclose(f);
	
	/* TCP wrappers stuff. */
	if (!(f = fopen("/harddisk/etc/hosts.deny", "w")))
	{
		errorbox("Unable to write /etc/hosts.deny.");
		return 0;
	}
	fprintf(f, "ALL : ALL\n");
	fclose(f);
	
	if (!(f = fopen("/harddisk/etc/hosts.allow", "w")))
	{
		errorbox("Unable to write /etc/hosts.allow.");
		return 0;
	}
	fprintf(f, "ALL : localhost\n");
	fprintf(f, "ALL : %s/%s\n", np->netaddress, np->netmask);
	fclose(f);
	
	/* Root password. */
	if (!(getpassword(password, "Enter the 'root' user password.")))
	{
		errorbox("'root' user password not given.");
		return 0;
	}
	
	snprintf(commandstring, STRING_SIZE,
		"/bin/echo 'root:%s' | /bin/chroot /harddisk /usr/sbin/chpasswd",
		password);
	if (runcommandwithstatus(commandstring, "Setting root password...."))
	{
		errorbox("Problem setting root password.");
		return 0;
	}
	
	/* SmoothWall admin password. */
	if (!(getpassword(password, "Enter SmoothWall administrator password.")))
	{
		errorbox("SmoothWall administrator password not given.");
		return 0;
	}
	
	snprintf(commandstring, STRING_SIZE,
		"/bin/chroot /harddisk /usr/bin/htpasswd -b /var/smoothwall/auth/users admin %s",
		password);
	if (runcommandwithstatus(commandstring, "Setting Smoothwall administartor password...."))
	{
		errorbox("Problem setting SmoothWall administrator password.");
		return 0;
	}

	snprintf(devnode, STRING_SIZE, "%s", dp->devnode);
	if (symlink(devnode, "/harddisk/dev/harddisk"))
	{
		errorbox("Unable to create symlink /dev/harddisk.");
		return 0;
	}
	snprintf(devnode, STRING_SIZE, "%s1", dp->devnode);
	if (symlink(devnode, "/harddisk/dev/harddisk1"))
	{
		errorbox("Unable to create symlink /dev/harddisk1.");
		return 0;
	}
	snprintf(devnode, STRING_SIZE, "%s2", dp->devnode);
	if (symlink(devnode, "/harddisk/dev/harddisk2"))
	{
		errorbox("Unable to create symlink /dev/harddisk2.");
		return 0;
	}
	
	return 1;
}

/* Taken from the cdrom one. */
int getpassword(char *password, char *text)
{
	char *values[] = {	NULL, NULL, NULL };	/* pointers for the values. */
	struct newtWinEntry entries[] =
	{ 
		{ "Password:", &values[0], 2 },
		{ "Again:", &values[1], 2 },
		{ NULL, NULL, 0 }
	};
	int rc;
	int done;
	
	do
	{
		done = 1;
		rc = newtWinEntries(TITLE, text,
			50, 5, 5, 20, entries, "OK", "Cancel", NULL);
		
		if (rc != 2)
		{
			if (strlen(values[0]) == 0 || strlen(values[1]) == 0)
			{
				errorbox("Password cannot be blank.");
				done = 0;
				strcpy(values[0], "");
				strcpy(values[1], "");				
			}
			else if (strcmp(values[0], values[1]) != 0)
			{
				errorbox("Passwords do not match.");
				done = 0;
				strcpy(values[0], "");
				strcpy(values[1], "");					
			}
		}
	}
	while (!done);

	strncpy(password, values[0], STRING_SIZE);

	if (values[0]) free(values[0]);
	if (values[1]) free(values[1]);

	return rc;
}
	
