/* SmoothWall install program.
 *
 * By The SmoothWall Team.
 * 
 * This program is distributed under the terms of the GNU General Public
 * Licence.  See the file COPYING for details.
 *
 * filename: nic.c
 * Contains stuff related to firing up the network card, including a crude
 * autodector. */

#include "install.h"

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

extern FILE *flog;
extern char *log;

static int getdrivername(char *driver);
static int getaddresses(struct networkparams *p);

/* One nic. */
struct nic
{
	char *description;
	char *modulename;
};

int networkmenu(struct networkparams *p)
{
	int done;
	int rc;
	char commandstring[STRING_SIZE];
	char messagestring[STRING_SIZE];
	struct nic nics[] = {
		{ "3Com EtherLink III", "3c509" },
		{ "3Com 3c501", "3c501" },
		{ "3Com ISA EtherLink XL", "3c515" },
		{ "3Com 3c503 and 3c503/16", "3c503" },
		{ "3Com EtherLink Plus (3c505)", "3c505" },
		{ "3Com EtherLink 16", "3c507" },
		{ "\"Corkscrew\" 3Com EtherLink PCI III/XL;Vortex (3c59x);Boomerage (3c90x, 3c595)", "3c59x" },
		{ "PCI NE2000 cards, RealTEk RTL-8029, Winbond 89C940, etc", "ne2k-pci" },
		{ "NE1000 / NE2000 (non-pci)", "ne" },
		{ "EtherWORKS DE425 TP/COAX EISA, DE434 TP PCI, DE435/450, DE500 10/100 PCI", "de4x5" },
		{ "Digital 21x4x Tulip PCI ethernet cards, etc.", "tulip" },
		{ "Intel i82557/i82558 PCI EtherExpressPro", "eepro100" },
		{ "Intel i82595 ISA EtherExpressPro10/10+ driver" ,"eepro" },
		{ "Intel EtherExpress 16 (i82586)", "eexpress" },
		{ "AMD LANCE/PCnetAllied Telesis AT1500, HP J2405A, NE2100, NE2500", "lance" },
		{ "AMD PCnet32 and AMD PCnetPCI", "pcnet32" },
		{ "Apricot 680x0 VME, 82596 chipset", "82596" },
		{ "Cabletron E2100 seies ethercards", "e2100" },
		{ "Crystal LAN CS8900/CS8920", "cs89x0" },
		{ "Compaq Netelligent 10/100 TX PCI UTP, etc", "tlan" },
		{ "100VG-AnyLan Network Adapters, HP J2585B, J2585A, J2970, J2973, J2573, etc", "hp100" },
		{ "VIA Rhine PCI Fast Ethernet, etc", "via-rhine" },
		{ "DIGITAL DEPCA & EtherWORKS,DEPCA, DE100, etc", "depca" },
		{ "Digi Intl. RightSwitch SE-X EISA and PCI", "dgrs" },
		{ "EtherWORKS 3 (DE203, DE204and DE205", "ewrk3" },
		{ "Fujitsu FMV-181/182/183/184", "fmv18x" },
		{ "HP PCLAN/plus", "hp-plus" },
		{ "HP LAN ethernet", "hp" },
		{ "ICL EtherTeam 16i/32" ,"eth16i" },
		{ "MiCom-Interlan NI5010 ethercard", "ni5010" },
		{ "NI5210 card (i82586 Ethernet chip)", "ni52" }, 
		{ "NI6510, ni6510 EtherBlaster", "ni65" },
		{ "RealTek cards using RTL8129or RTL8139 Fast Ethernet chipsets", "rtl8139" },
		{ "SMC Ultra and SMC EtherEZ ISA ethercard (8K, 83c790)", "smc-ultra" },
		{ "SMC Ultra32 EISA ethernet card (32K)", "smc-ultra32" },
		{ "SMC 9000 series of ethernet cards", "smc9194" },
		{ "PureData PDUC8028,WD8003 and WD8013 \"compatible\" ethercards", "wd" },
		{ "SiS 900 PCI", "sis900" },
		{ NULL, NULL }
	};
	int niccount = 0;
	
	/* Clear the name.  If this is clear, we known detect failed. */
	strcpy(p->modulename, "");

	/* Loop through all cards in the nic array, trying to load each one. */	
	while (nics[niccount].description)
	{
		snprintf(messagestring, STRING_SIZE, "Looking for: %s", nics[niccount].description);
		snprintf(commandstring, STRING_SIZE, "/sbin/modprobe %s", nics[niccount].modulename);

		if (!(runcommandwithstatus(commandstring, messagestring)))
		{
			strncpy(p->modulename, nics[niccount].modulename, STRING_SIZE);
			snprintf(messagestring, STRING_SIZE, "SmoothWall has detected the following " \
				"NIC in your machine: %s", nics[niccount]. description);
			newtWinMessage(TITLE, "Ok",	messagestring);
			break;
		}
		niccount++;
	}
	
	if (!strlen(p->modulename))
	{
		done = 0;
		while (!done)
		{
			/* Get driver and params. */
			rc = getdrivername(p->modulename);
			if (rc != 1)
				return 0;
		
			snprintf(commandstring, STRING_SIZE, "/sbin/modprobe %s", p->modulename);
			if (!(mysystem(commandstring)))
				done = 1;
			else
				errorbox("Module failed to load.");
		}
	}	

	/* Get IP address info. */
	rc = getaddresses(p);
	if (rc != 1)
		return 0;

	snprintf(commandstring, STRING_SIZE,
		"/cdrom/bin/ifconfig eth0 %s netmask %s up", p->address, p->netmask);
	if (!(mysystem(commandstring)))
		done = 1;
	else
	{
		errorbox("Interface failed to come up.");
		return 0;
	}
			
	return 1;
}
			

static int getdrivername(char *driver)
{
	char *values[] = {	NULL, NULL };	/* pointers for the values. */
	struct newtWinEntry entries[] =
		{ { "", &values[0], 0,}, { NULL, NULL, 0 } };
	int rc;
	
	rc = newtWinEntries(TITLE, "Failed to detect a network card automatically. " \
		"Enter the driver and optional parameters for the network card.",
		 50, 5, 5, 20, entries, "OK", "Cancel", NULL);
		
	strncpy(driver, values[0], STRING_SIZE);
	
	return rc;
}

static int getaddresses(struct networkparams *p)
{
	char *values[] = {	NULL, NULL, NULL, NULL};	/* pointers for the values. */
	struct newtWinEntry entries[] =
	{ 
		{ "IP address", &values[0], 0, },
		{ "Network address", &values[1], 0 },
		{ "Network mask", &values[2], 0, }, 
		{ NULL, NULL, 0 }
	};
	int done;
	int rc;
	int c;
	char message[STRING_SIZE];

	/* Simular sort of stuff to the CDROM menu. */
	do
	{
		done = 1;
	
		rc = newtWinEntries(TITLE, "Enter the IP address information", 50, 5, 5,
			18, entries, "OK", "Cancel", NULL);

		if (rc != 2)
		{
			strcpy(message, "The following fields are invalid:\n\n");
			if (inet_addr(values[0]) == INADDR_NONE)
			{
				strcat(message, "IP address\n");
				done = 0;
			}
			if (inet_addr(values[1]) == INADDR_NONE )
			{
				strcat(message, "Network address\n");
				done = 0;
			}

			if (inet_addr(values[2]) == INADDR_NONE)
			{
				strcat(message, "Network mask\n");
				done = 0;
			}

			if (!done)
				errorbox(message);
		}	
	}
	while (!done);

	strncpy(p->address, values[0], STRING_SIZE);
	strncpy(p->netaddress, values[1], STRING_SIZE);
	strncpy(p->netmask, values[2], STRING_SIZE);
	
	for (c = 0; c < 3; c++)
	{
		if (values[c]) free(values[c]);
	}
	
	return rc;
}
	
